import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class VFDScreen 
  extends JPanel {

  // dimensions, in pixels
  private int dotSize_       = 3;
  private int dotSpacing_    = 1;
  private int underlineSize_ = 2;
  private int sideBorder_    = 2;
  private int topBorder_     = 2; // also bottom border

  private Color dotColor100_ = new Color(0, 255, 255);
  private Color dotColor75_  = new Color(0, 210, 210);
  private Color dotColor50_  = new Color(0, 170, 170);
  private Color dotColor25_  = new Color(0, 130, 130);
  private Color dotColor0_   = new Color(0, 0, 0);
  private Color bgColor_     = Color.black;

  private Vector els_; // of VFDChars
  private int height_; // in characters
  private int width_;

  private int brightness_; // 0-100 pct
  private boolean isOn_;

  public VFDScreen(int width, int height) {
    width_ = width;
    height_ = height;
    els_ = new Vector(width_ * height_);
    for (int i = 0; i < width_ * height_; i++) 
      els_.add(new VFDChar());

    Dimension d = new Dimension(pixelWidth(), pixelHeight());
    setPreferredSize(d);
    setMinimumSize(d);
    setMaximumSize(d);

    brightness_ = 100;
    isOn_ = true;
  }

  public void setOn(boolean on) { isOn_ = on; }

  public void setBrightness(int pct) {
    Util.assert(pct >= 0 && pct <= 100);
    brightness_ = pct;
  }

  public VFDChar getChar(int x, int y) {
    Util.assert(x >= 0 && x < charWidth());
    Util.assert(y >= 0 && y < charHeight());
    return (VFDChar) els_.get(x + y * charWidth());
  }
 
  public void setChar(int x, int y, VFDChar c) {
    Util.assert(x >= 0 && x < charWidth());
    Util.assert(y >= 0 && y < charHeight());
    els_.setElementAt(c, x + y * charWidth());
  }
 
  // character element dimension
  public int charHeight() { return height_; }
  public int charWidth()  { return width_; }

  public int charPixelHeight() {
    int h = VFDChar.height() * (dotSize_ + dotSpacing_) - dotSpacing_;
    h += dotSpacing_ + underlineSize_; // make room for underline
    h += 2 * topBorder_;
    return h;
  }

  public int charPixelWidth() {
    int w = VFDChar.width() * (dotSize_ + dotSpacing_) - dotSpacing_;
    w += 2 * sideBorder_;
    return w;
  }

  public int pixelHeight() { return charHeight() * charPixelHeight(); }
  public int pixelWidth()  { return charWidth()  * charPixelWidth();  }

  // place pixel offset of char at x,y into d
  public void getOffsetOf(int x, int y, Dimension d) {
    Util.assert(x >= 0 && x < charWidth());
    Util.assert(y >= 0 && y < charHeight());
    int px = x * charPixelWidth();
    int py = y * charPixelHeight();
    d.setSize(px, py);
  }

  private Color getDotColor(int brightness) {
    if (brightness > 75) return dotColor100_;
    else if (brightness > 50 && brightness <= 75) return dotColor75_;
    else if (brightness > 25 && brightness <= 50) return dotColor50_;
    else if (brightness > 0 && brightness <= 25) return dotColor25_;
    else return dotColor0_;
  }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);

    // paint background
    g.setColor(bgColor_);
    g.fillRect(0, 0, pixelWidth(), pixelHeight());

    if (!isOn_)
      return;

    // paint characters
    g.setColor(getDotColor(brightness_));
    Dimension offset = new Dimension();
    for (int y = 0; y < charHeight(); y++) {
      for (int x = 0; x < charWidth(); x++) {
	getOffsetOf(x, y, offset);
	VFDChar c = getChar(x, y);
	if (c != null)
	  paintCharAt(g, offset, c);
      }
    }
  }

  private void paintCharAt(Graphics g, Dimension offset, VFDChar c) {
    int y = offset.height + topBorder_;
    int x0 = offset.width + sideBorder_;
    int x = x0;
    for (int row = 0; row < VFDChar.height(); row++) {
      x = x0;
      for (int col = 0; col < VFDChar.width(); col++) {
	if (c.getDot(col, row)) 
	  g.fillOval(x, y, dotSize_, dotSize_);
	x += dotSize_ + dotSpacing_;
      }
      y += dotSize_ + dotSpacing_;
    }
    x = x0;
    if (c.isUnderlined()) 
      g.fillRect(x, y, VFDChar.width()*(dotSize_ + dotSpacing_) - dotSpacing_, underlineSize_);
  }

  public static void main(String args[]) {

    VFDCharFactory cf = new VFDCharFactory();
    try { cf.loadFontFromFile("chars.txt"); }
    catch (IOException x) { Util.fail("Couldn't read font file"); }
    

    VFDScreen screen = new VFDScreen(40, 2);
    for (int x = 0; x < 40; x++)
      for (int y = 0; y < 2; y++) 
	screen.getChar(x, y).setAllDots();
    
    int first = 0xe0;
    for (int x = first; x <= 0xff; x++) {
      VFDChar c = cf.getCharForCode(x);
      if (c != null && x - first < 40)
	screen.setChar(x - first, 0, c);
    }

    if (args.length > 0)
      screen.setBrightness(Integer.parseInt(args[0]));

    JFrame frame = new JFrame("VFDScreen Test");
    frame.getContentPane().add(screen, BorderLayout.CENTER);
    frame.addWindowListener(
      new WindowAdapter() {
	  public void windowClosing(WindowEvent e) {
	      System.exit(0);
	  } } );
    frame.pack();
    frame.setVisible(true);
  }
}
